# mkccharts.tcl
# generates html for viewing/comparing POV-Ray/X Window System colours.

set gamma 2.2

set incfiles(pov) /usr/local/share/povray-3.7/include/colors.inc
set incfiles(x11) ./colours_x11.inc

# -----------------------------------------------------------------------------
# call povray to do the colour swatches.

proc render {which name} {
  global $which gamma

  set fn ${name}.pov
  set n [expr {[llength [set ${which}(c)]] + [llength [set ${which}(g)]]}]
  set cmd [list exec -ignorestderr -- nice povray Display_Gamma=sRGB]
  lappend cmd +W100 +H20 +A0.3 [format +AG%s $gamma] +Q9 -X -D -GA -P -V
  if { "pov" == $which } {
    lappend cmd Declare=X11=0 [format +O%s/img/p_.png $name]
  } else {
    lappend cmd +L. Declare=X11=1 [format +O%s/img/x_.png $name]
  }
  lappend cmd [format +KFF%d $n] [format +I%s $fn]

  {*}$cmd

  return
}


# -----------------------------------------------------------------------------
# retrieve the relevant lines from each .inc file, remove *grey*s and guards,
# return a two-element array of (sorted) colours/grays lists.

proc ldelete {lVar v} {
  upvar $lVar list
  set i [lsearch -exact $list $v]
  set list [lreplace $list $i $i]
}

set sed(pov) [list {sed -ne "1,/\\/\\/ Color manipulation macros/p"} Colors_Inc_Temp]
set sed(x11) [list {sed -ne "/#declare /p"} colours_x11_include_temp]

proc getData {arrVar which} {
  global incfiles sed

  upvar $arrVar arr

  set fp [open [format {|%s %s} [lindex $sed($which) 0] $incfiles($which)] r]
  set raw [split [read $fp] \n]
  close $fp

  set re_gray {^gray[[:digit:]]+}
  set re_line {#declare\s+(\w+)\s+=\s.*}
  set C [list] ; set G [list]
  foreach line $raw {
    if { ![regexp $re_line $line -> name] } {
      continue
    } elseif { [string match -nocase *grey* $name] } {
      continue
    } elseif { [regexp -nocase $re_gray $name] } {
      lappend G $name
    } else {
      lappend C $name
    }
  }
  ldelete C [lindex $sed($which) 1]
  set arr(c) [lsort -dictionary $C] ; set arr(g) [lsort -dictionary $G]

  return
}


# -----------------------------------------------------------------------------
# emit the POV and X11 stuff (2-column tables).  insert an empty row between
# colours and grays for visual separation.

proc article {which h2 incfn fp} {
  global $which

  set C [set ${which}(c)] ; set G [set ${which}(g)]

  puts $fp [format {  <article>
    <h2><a name='%s'>%s</a></h2>
    <p>file '%s' provides %d colours and %d grays. (duplicates, ie *grey*, removed)</p>
    <table>} \
          [format tbl_%s $which] $h2 [file tail $incfn] [llength $C] [llength $G]]

  if { "pov" == $which } { set fmt {img/p_%03d.png} } { set fmt {img/x_%03d.png} }
  set iid 1
  foreach name $C {
    puts $fp [format {      <tr><td>%s</td><td><img src='%s' alt='%s'/></td></tr>} \
            $name [format $fmt $iid] $name]
    incr iid
  }
  puts $fp {      <tr><td></td><td></td></tr>}
  foreach name $G {
    puts $fp [format {      <tr><td>%s</td><td><img src='%s' alt='%s'/></td></tr>} \
            $name [format $fmt $iid] $name]
    incr iid
  }

  puts $fp {    </table>
    <footer>[<a href='#tbl_top'>back to top</a>]</footer>
  </article>}

  return
}


# -----------------------------------------------------------------------------
# emit the comparison table (3-columns).

proc articleCmp {fp} {
  global pov x11

  # common colours count is needed before table, so build list of pairs.
  set todo [list]
  set i 1
  foreach c $pov(c) {
    set j [lsearch -sorted -exact $x11(c) $c]
    if { -1 != $j } {
      incr j
      lappend todo [list $i $j]
    }
    incr i
  }

  set fmtp {img/p_%03d.png} ; set fmtx {img/x_%03d.png}

  puts $fp [format {  <article>
    <h2><a name='tbl_cmp'>comparison</a></h2>
    <p>the above include files have %d colour names in common, but few result in the same colours. (GrayNN's excluded)</p>
    <table>
      <tr><th></th><th>POV-Ray</th><th>X11</th></tr>} [llength $todo]]

  foreach p $todo {
    set name [lindex $pov(c) [expr {[lindex $p 0] - 1}]]
    puts $fp [format {      <tr><td>%s</td><td><img src='%s' alt='%s'/></td><td><img src='%s' alt='%s'/></td></tr>} \
            $name [format $fmtp [lindex $p 0]] $name [format $fmtx [lindex $p 1]] $name]
  }

  puts $fp {    </table>
    <footer>[<a href='#tbl_top'>back to top</a>]</footer>
    <br/>
  </article>}

  return
}


# -----------------------------------------------------------------------------
# generate the array file(s).

proc writeArray {which guard asize aname fp} {
  global $which

  # colours and grays as one.
  set todo [join [list [set ${which}(c)] [set ${which}(g)]]]
  set n [llength $todo]

  puts $fp [format {// cchart temporary array
#ifndef(%s)
#declare %s = 1;} $guard $guard]

  puts $fp [format {#declare %s = %d;
#declare %s = array[%d] %s} $asize $n $aname $n "\{"]

  foreach item $todo {
    set fmt {  %s}
    if { [incr n -1] } { append fmt , }
    puts $fp [format $fmt $item]
  }

  puts $fp [format {%s
#end} "\}"]

  return
}


# ---------------------------------------------------------------<PROGRAM>-----

set ident ccharts

# bail if run already.
if { [file isdirectory $ident] } { exit 1 }

foreach name [array names incfiles] { getData $name $name }

# output goes into sub-directory and descendant.
set cssfn ${ident}.css
file mkdir ${ident}/img
file copy $cssfn ${ident}

foreach name [array names incfiles] {
  set fp [open ${ident}.array w]
  writeArray $name ${ident}_include_temp ${ident}Size ${ident}Colour $fp
  close $fp
  render $name $ident
  file delete ${ident}.array
}

# build html page.
set outfile ${ident}/index.html
set fp [open $outfile w]

puts $fp [format {<!DOCTYPE html>
<html lang='en'>
<head>
  <meta charset='utf8'/>
  <title>colour charts</title>
  <link rel='stylesheet' type='text/css' href='%s'/>
</head>
<body>
  <header>
    <h1><a name='tbl_top'>POV-Ray &amp; X Window System colours</a></h1>
    <nav>
      <ul>
        <li><a href='#tbl_pov'>POV-Ray</a></li>
        <li><a href='#tbl_x11'>X11</a></li>
        <li><a href='#tbl_cmp'>comparison</a></li>
      </ul>
    </nav>
  </header>} $cssfn]

foreach name [array names incfiles] title [list POV-Ray X11] {
  article $name $title $incfiles($name) $fp
}

articleCmp $fp

puts $fp {  <hr/>
  <footer><small>copyright (c) 2016 jr.</small></footer>
</body>
</html>}

close $fp

# feedback.
puts [format {created colours charts page '%s'.} $outfile]

exit 0

